@@ -1,10 +1,10 @@
 from .ws_client import WebSockClient
 import pickle
-from .reddit_auth import _RedditAuthBase, TokenAuth, PasswordAuth
+from .reddit_auth import TokenAuth, PasswordAuth
 from websocket import WebSocketConnectionClosedException
 from ._api.tools import Tools
-from ._api.models import Channel, Message
-from typing import Dict, List, Optional, Callable
+from ._api.models import Channel, Message, BannedUsers
+from typing import Dict, List, Optional, Callable, Union
 from ._utils.frame_model import FrameType, FrameModel
 from ._events import Events
 
@@ -18,7 +18,8 @@ _hook = Callable[[FrameModel], Optional[bool]]
 
 
 class ChatBot:
-    def __init__(self, authentication: _RedditAuthBase, store_session: bool = True, log_error_frames=True, **kwargs):
+    def __init__(self, authentication: Union[TokenAuth, PasswordAuth], store_session: bool = True,
+                 log_error_frames=True, **kwargs):
         self.__r_authentication = authentication
         self._store_session = store_session
         if store_session:
@@ -124,8 +125,8 @@ class ChatBot:
     def send_snoomoji(self, snoomoji: str, channel_url: str) -> None:
         self.__WebSocketClient.ws_send_snoomoji(snoomoji, channel_url)
 
-    def send_gif(self, gif_url: str, channel_url: str) -> None:
-        self.__WebSocketClient.ws_send_gif(gif_url, channel_url)
+    def send_gif(self, gif_url: str, channel_url: str, height: int = 200, width: int = 200) -> None:
+        self.__WebSocketClient.ws_send_gif(gif_url, channel_url, height, width)
 
     def send_typing_indicator(self, channel_url: str) -> None:
         self.__WebSocketClient.ws_send_typing_indicator(channel_url)
@@ -134,21 +135,22 @@ class ChatBot:
         self.__WebSocketClient.ws_stop_typing_indicator(channel_url)
 
     def run_4ever(self, auto_reconnect: bool = True, max_retries: int = 500, disable_ssl_verification: bool = False,
-                  skip_utf8_validation=True, **kwargs) -> None:
+                  skip_utf8_validation: bool = True, **kwargs) -> None:
         if disable_ssl_verification:
             import ssl
             sslopt = {"cert_reqs": ssl.CERT_NONE}
         else:
             sslopt = None
 
+        self.__tools._is_running = True
         for _ in range(max_retries):
             self.__WebSocketClient.ws.run_forever(ping_interval=15, ping_timeout=5,
                                                   skip_utf8_validation=skip_utf8_validation,
                                                   sslopt=sslopt,
-                                                  **kwargs
-                                                  # ping_payload="{active:1}"
+                                                  **kwargs,
+                                                  ping_payload="{active:1}"
                                                   )
-            if self.__WebSocketClient.is_logi_err and isinstance(self.__r_authentication, PasswordAuth):
+            if self.__WebSocketClient.is_logi_err and self.__r_authentication.is_reauthable:
                 self.__WebSocketClient.logger.info("Re-Authenticating...")
                 if self._store_session:
                     sb_access_token, _ = self._load_session(self.__r_authentication.reddit_username, force_reauth=True)
@@ -166,11 +168,11 @@ class ChatBot:
     def kick_user(self, channel_url: str, user_id: str, duration: int) -> None:
         self.__tools.kick_user(**_get_locals_without_self(locals()))
 
-    def delete_mesg(self, channel_url: str, msg_id: str) -> None:
+    def delete_mesg(self, channel_url: str, msg_id: int) -> None:
         self.__tools.delete_message(**_get_locals_without_self(locals()),
                                     session_key=self.__WebSocketClient.session_key)
 
-    def invite_user_to_channel(self, channel_url: str, nicknames: List[str]) -> None:
+    def invite_user_to_channel(self, channel_url: str, nicknames: Union[str, List[str]]) -> None:
         self.__tools.invite_user(**_get_locals_without_self(locals()))
 
     def leave_chat(self, channel_url: str):
@@ -182,20 +184,28 @@ class ChatBot:
     def get_channels(self, limit: int = 100, order: str = "latest_last_message", show_member: bool = True,
                      show_read_receipt: bool = True, show_empty: bool = True, member_state_filter: str = "joined_only",
                      super_mode: str = "all", public_mode: str = "all", unread_filter: str = "all",
-                     hidden_mode: str = "unhidden_only", show_frozen: bool = True) -> List[Channel]:
+                     hidden_mode: str = "unhidden_only", show_frozen: bool = True,
+                     # custom_types: str = 'direct,group'
+                     ) -> List[Channel]:
         return self.__tools.get_channels(**_get_locals_without_self(locals()),
                                          session_key=self.__WebSocketClient.session_key)
 
     def get_members(self, channel_url: str, next_token: str = None, limit: int = 20,
-                    order: str = "member_nickname_alphabetical", member_state_filter: str = "all"):
+                    order: str = "member_nickname_alphabetical", member_state_filter: str = "all",
+                    nickname_startswith: str = ''):
         return self.__tools.get_members(**_get_locals_without_self(locals()),
                                         session_key=self.__WebSocketClient.session_key)
 
+    def get_banned_members(self, channel_url, limit: int = 100) -> BannedUsers:
+        return self.__tools.get_banned_members(**_get_locals_without_self(locals()),
+                                               session_key=self.__WebSocketClient.session_key)
+
     def get_current_channels(self) -> List[Channel]:
         return self.__WebSocketClient.current_channels
 
-    def get_older_messages(self, channel_url: str, message_ts: int = 9007199254740991, prev_limit: int = 40,
-                           next_limit: int = 0, reverse: bool = True) -> List[Message]:
+    def get_older_messages(self, channel_url: str, message_ts: Union[int, str] = 9007199254740991,
+                           custom_types: str = '*', prev_limit: int = 40, next_limit: int = 0,
+                           reverse: bool = True) -> List[Message]:
         return self.__tools.get_older_messages(**_get_locals_without_self(locals()),
                                                session_key=self.__WebSocketClient.session_key)
 
@@ -225,6 +235,20 @@ class ChatBot:
                   allow_auto_unhide: bool = True) -> None:
         self.__tools.hide_chat(**_get_locals_without_self(locals()), session_key=self.__WebSocketClient.session_key)
 
+    def mute_user(self, channel_url: str, user_id: str, duration: int, description: str):
+        self.__tools.mute_user(**_get_locals_without_self(locals()), session_key=self.__WebSocketClient.session_key)
+
+    def unmute_user(self, channel_url: str, user_id: str):
+        self.__tools.unmute_user(**_get_locals_without_self(locals()), session_key=self.__WebSocketClient.session_key)
+
+    def set_channel_frozen_status(self, channel_url: str, is_frozen: bool):
+        self.__tools.set_channel_frozen_status(**_get_locals_without_self(locals()),
+                                               session_key=self.__WebSocketClient.session_key)
+
+    def delete_channel(self, channel_url: str):
+        self.__tools.delete_channel(**_get_locals_without_self(locals()),
+                                    session_key=self.__WebSocketClient.session_key)
+
     def enable_rate_limiter(self, max_calls: float, period: float) -> None:
         self.__WebSocketClient.RateLimiter.is_enabled = True
         self.__WebSocketClient.RateLimiter.max_calls = max_calls
